﻿using Microsoft.Win32;
using System;
using System.Threading;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 경고 폼 보여주기 대리자 - ShowWarningFormDelegate(message)

        /// <summary>
        /// 경고 폼 보여주기 대리자
        /// </summary>
        /// <param name="message">메시지</param>
        private delegate void ShowWarningFormDelegate(string message);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 레지스트리 키
        /// </summary>
        private RegistryKey registryKey = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\services\USBSTOR", true);

        /// <summary>
        /// 경고 폼 보여주기 대리자
        /// </summary>
        private ShowWarningFormDelegate showWarningFormDelegate = null;

        /// <summary>
        /// 스레드
        /// </summary>
        private Thread thread = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 닫을 경우 처리하기 - Form_FormClosing(sender, e)

        /// <summary>
        /// 폼 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_FormClosing(object sender, FormClosingEventArgs e)
        {
            if(this.thread != null)
            {
                this.thread.Abort();
            }

            Application.ExitThread();
        }

        #endregion
        #region 적용 버튼 클릭시 처리하기 - applyButton_Click(sender, e)

        /// <summary>
        /// 적용 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void applyButton_Click(object sender, EventArgs e)
        {
            if(this.enableUSBPortCheckBox.Checked == true)
            {
                this.registryKey.SetValue("Start", (int)Permission.ALLOW);

                if(this.thread != null)
                {
                    this.thread.Abort();
                }
            }
            else if(this.disableUSBPortCheckBox.Checked == true)
            {
                this.registryKey.SetValue("Start", (int)Permission.DENY);

                this.showWarningFormDelegate = new ShowWarningFormDelegate(ShowWarningForm);

                if(this.realTimeCheckBox.Checked == true)
                {
                    this.thread = new Thread(ProcessThread);

                    this.thread.Start();
                }
            }
        }

        #endregion
        #region USB 포트 활성화 체크 박스 체크 변경시 처리하기 - enableUSBPortCheckBox_CheckedChanged(sender, e)

        /// <summary>
        /// USB 포트 
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void enableUSBPortCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.realTimeCheckBox.Checked = false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 스레드 처리하기 - ProcessThread()

        /// <summary>
        /// 스레드 처리하기
        /// </summary>
        private void ProcessThread()
        {
            while(true)
            {
                if((int)registryKey.GetValue("Start") != 4)
                {
                    this.registryKey.SetValue("Start", (int)Permission.DENY);

                    Invoke(this.showWarningFormDelegate, "레지스트리 비정상 접근 차단");
                }
            }
        }

        #endregion
        #region 경고 폼 보여주기 - ShowWarningForm(message)

        /// <summary>
        /// 경고 폼 보여주기
        /// </summary>
        /// <param name="message">메시지</param>
        private void ShowWarningForm(string message)
        {
            WarningForm form = new WarningForm();

            form.Message = message;

            form.ShowDialog();
        }

        #endregion
    }
}